@@ -1,7 +1,7 @@
 /******************************************************************************
  *
  * Project:  PROJ
- * Purpose:  ISO19111:2018 implementation
+ * Purpose:  ISO19111:2019 implementation
  * Author:   Even Rouault <even dot rouault at spatialys dot com>
  *
  ******************************************************************************
@@ -51,7 +51,7 @@ namespace cs {
 /** \brief The direction of positive increase in the coordinate value for a
  * coordinate system axis.
  *
- * \remark Implements AxisDirection from \ref ISO_19111_2018
+ * \remark Implements AxisDirection from \ref ISO_19111_2019
  */
 class AxisDirection : public util::CodeList {
   public:
@@ -119,7 +119,7 @@ using MeridianNNPtr = util::nn<MeridianPtr>;
 /** \brief The meridian that the axis follows from the pole, for a coordinate
  * reference system centered on a pole.
  *
- * \note There is no modelling for this concept in \ref ISO_19111_2018
+ * \note There is no modelling for this concept in \ref ISO_19111_2019
  *
  * \remark Implements MERIDIAN from \ref WKT2
  */
@@ -163,10 +163,10 @@ using CoordinateSystemAxisNNPtr = util::nn<CoordinateSystemAxisPtr>;
 
 /** \brief The definition of a coordinate system axis.
  *
- * \remark Implements CoordinateSystemAxis from \ref ISO_19111_2018
+ * \remark Implements CoordinateSystemAxis from \ref ISO_19111_2019
  */
-class PROJ_GCC_DLL CoordinateSystemAxis final
-    : public common::IdentifiedObject {
+class PROJ_GCC_DLL CoordinateSystemAxis final : public common::IdentifiedObject,
+                                                public io::IJSONExportable {
   public:
     //! @cond Doxygen_Suppress
     PROJ_DLL ~CoordinateSystemAxis() override;
@@ -193,7 +193,8 @@ class PROJ_GCC_DLL CoordinateSystemAxis final
         _isEquivalentTo(
             const util::IComparable *other,
             util::IComparable::Criterion criterion =
-                util::IComparable::Criterion::STRICT) const override;
+                util::IComparable::Criterion::STRICT,
+            const io::DatabaseContextPtr &dbContext = nullptr) const override;
 
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter, int order,
                                     bool disableAbbrev) const;
@@ -201,6 +202,9 @@ class PROJ_GCC_DLL CoordinateSystemAxis final
     PROJ_INTERNAL void _exportToWKT(io::WKTFormatter *formatter)
         const override; // throw(io::FormattingException)
 
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
     PROJ_INTERNAL static std::string normalizeAxisName(const std::string &str);
 
     PROJ_INTERNAL static CoordinateSystemAxisNNPtr
@@ -233,9 +237,10 @@ class PROJ_GCC_DLL CoordinateSystemAxis final
  * The coordinate values in a coordinate tuple shall be recorded in the order
  * in which the coordinate system axes associations are recorded.
  *
- * \remark Implements CoordinateSystem from \ref ISO_19111_2018
+ * \remark Implements CoordinateSystem from \ref ISO_19111_2019
  */
-class PROJ_GCC_DLL CoordinateSystem : public common::IdentifiedObject {
+class PROJ_GCC_DLL CoordinateSystem : public common::IdentifiedObject,
+                                      public io::IJSONExportable {
   public:
     //! @cond Doxygen_Suppress
     PROJ_DLL ~CoordinateSystem() override;
@@ -251,12 +256,16 @@ class PROJ_GCC_DLL CoordinateSystem : public common::IdentifiedObject {
         _exportToWKT(io::WKTFormatter *formatter)
             const override; // throw(io::FormattingException)
 
+    PROJ_INTERNAL void _exportToJSON(io::JSONFormatter *formatter)
+        const override; // throw(FormattingException)
+
     PROJ_INTERNAL virtual std::string getWKT2Type(bool) const = 0;
 
-    PROJ_INTERNAL bool
-    _isEquivalentTo(const util::IComparable *other,
-                    util::IComparable::Criterion criterion =
-                        util::IComparable::Criterion::STRICT) const override;
+    PROJ_INTERNAL bool _isEquivalentTo(
+        const util::IComparable *other,
+        util::IComparable::Criterion criterion =
+            util::IComparable::Criterion::STRICT,
+        const io::DatabaseContextPtr &dbContext = nullptr) const override;
     //! @endcond
 
   protected:
@@ -289,7 +298,7 @@ using SphericalCSNNPtr = util::nn<SphericalCSPtr>;
  * ellipsoid "degenerated" into a sphere. A SphericalCS shall have three
  * axis associations.
  *
- * \remark Implements SphericalCS from \ref ISO_19111_2018
+ * \remark Implements SphericalCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL SphericalCS final : public CoordinateSystem {
   public:
@@ -305,6 +314,11 @@ class PROJ_GCC_DLL SphericalCS final : public CoordinateSystem {
            const CoordinateSystemAxisNNPtr &axis2,
            const CoordinateSystemAxisNNPtr &axis3);
 
+    PROJ_DLL static SphericalCSNNPtr
+    create(const util::PropertyMap &properties,
+           const CoordinateSystemAxisNNPtr &axis1,
+           const CoordinateSystemAxisNNPtr &axis2);
+
   protected:
     PROJ_INTERNAL explicit SphericalCS(
         const std::vector<CoordinateSystemAxisNNPtr> &axisIn);
@@ -332,7 +346,7 @@ using EllipsoidalCSNNPtr = util::nn<EllipsoidalCSPtr>;
  *
  * An EllipsoidalCS shall have two or three associations.
  *
- * \remark Implements EllipsoidalCS from \ref ISO_19111_2018
+ * \remark Implements EllipsoidalCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL EllipsoidalCS final : public CoordinateSystem {
   public:
@@ -345,19 +359,27 @@ class PROJ_GCC_DLL EllipsoidalCS final : public CoordinateSystem {
     create(const util::PropertyMap &properties,
            const CoordinateSystemAxisNNPtr &axis1,
            const CoordinateSystemAxisNNPtr &axis2);
+
     PROJ_DLL static EllipsoidalCSNNPtr
     create(const util::PropertyMap &properties,
            const CoordinateSystemAxisNNPtr &axis1,
            const CoordinateSystemAxisNNPtr &axis2,
            const CoordinateSystemAxisNNPtr &axis3);
+
     PROJ_DLL static EllipsoidalCSNNPtr
     createLatitudeLongitude(const common::UnitOfMeasure &unit);
+
     PROJ_DLL static EllipsoidalCSNNPtr createLatitudeLongitudeEllipsoidalHeight(
         const common::UnitOfMeasure &angularUnit,
         const common::UnitOfMeasure &linearUnit);
+
     PROJ_DLL static EllipsoidalCSNNPtr
     createLongitudeLatitude(const common::UnitOfMeasure &unit);
 
+    PROJ_DLL static EllipsoidalCSNNPtr createLongitudeLatitudeEllipsoidalHeight(
+        const common::UnitOfMeasure &angularUnit,
+        const common::UnitOfMeasure &linearUnit);
+
     //! @cond Doxygen_Suppress
 
     /** \brief Typical axis order. */
@@ -411,7 +433,7 @@ using VerticalCSNNPtr = util::nn<VerticalCSPtr>;
  * Such a coordinate system is usually dependent on the Earth's gravity field.
  * A VerticalCS shall have one axis association.
  *
- * \remark Implements VerticalCS from \ref ISO_19111_2018
+ * \remark Implements VerticalCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL VerticalCS final : public CoordinateSystem {
   public:
@@ -460,7 +482,7 @@ using CartesianCSNNPtr = util::nn<CartesianCSPtr>;
  * three axis associations; the number of associations shall equal the
  * dimension of the CS.
  *
- * \remark Implements CartesianCS from \ref ISO_19111_2018
+ * \remark Implements CartesianCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL CartesianCS final : public CoordinateSystem {
   public:
@@ -529,7 +551,7 @@ using OrdinalCSNNPtr = util::nn<OrdinalCSPtr>;
  * The number of associations shall equal the
  * dimension of the CS.
  *
- * \remark Implements OrdinalCS from \ref ISO_19111_2018
+ * \remark Implements OrdinalCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL OrdinalCS final : public CoordinateSystem {
   public:
@@ -565,7 +587,7 @@ using ParametricCSNNPtr = util::nn<ParametricCSPtr>;
 /** \brief one-dimensional coordinate reference system which uses parameter
  * values or functions that may vary monotonically with height.
  *
- * \remark Implements ParametricCS from \ref ISO_19111_2018
+ * \remark Implements ParametricCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL ParametricCS final : public CoordinateSystem {
   public:
@@ -603,7 +625,7 @@ using TemporalCSNNPtr = util::nn<TemporalCSPtr>;
  *
  * A TemporalCS shall have one axis association.
  *
- * \remark Implements TemporalCS from \ref ISO_19111_2018
+ * \remark Implements TemporalCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL TemporalCS : public CoordinateSystem {
   public:
@@ -616,7 +638,7 @@ class PROJ_GCC_DLL TemporalCS : public CoordinateSystem {
     INLINED_MAKE_SHARED
 
     PROJ_INTERNAL std::string
-    getWKT2Type(bool use2018Keywords) const override = 0;
+    getWKT2Type(bool use2019Keywords) const override = 0;
 
   private:
     TemporalCS(const TemporalCS &other) = delete;
@@ -637,7 +659,7 @@ using DateTimeTemporalCSNNPtr = util::nn<DateTimeTemporalCSPtr>;
  * axisUnitID; the temporal quantities are defined through the ISO 8601
  * representation.
  *
- * \remark Implements DateTimeTemporalCS from \ref ISO_19111_2018
+ * \remark Implements DateTimeTemporalCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL DateTimeTemporalCS final : public TemporalCS {
   public:
@@ -654,7 +676,7 @@ class PROJ_GCC_DLL DateTimeTemporalCS final : public TemporalCS {
         const CoordinateSystemAxisNNPtr &axis);
     INLINED_MAKE_SHARED
 
-    PROJ_INTERNAL std::string getWKT2Type(bool use2018Keywords) const override;
+    PROJ_INTERNAL std::string getWKT2Type(bool use2019Keywords) const override;
 
   private:
     DateTimeTemporalCS(const DateTimeTemporalCS &other) = delete;
@@ -673,7 +695,7 @@ using TemporalCountCSNNPtr = util::nn<TemporalCountCSPtr>;
  *
  * A TemporalCountCS shall have one axis association.
  *
- * \remark Implements TemporalCountCS from \ref ISO_19111_2018
+ * \remark Implements TemporalCountCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL TemporalCountCS final : public TemporalCS {
   public:
@@ -690,7 +712,7 @@ class PROJ_GCC_DLL TemporalCountCS final : public TemporalCS {
         const CoordinateSystemAxisNNPtr &axis);
     INLINED_MAKE_SHARED
 
-    PROJ_INTERNAL std::string getWKT2Type(bool use2018Keywords) const override;
+    PROJ_INTERNAL std::string getWKT2Type(bool use2019Keywords) const override;
 
   private:
     TemporalCountCS(const TemporalCountCS &other) = delete;
@@ -709,7 +731,7 @@ using TemporalMeasureCSNNPtr = util::nn<TemporalMeasureCSPtr>;
  *
  * A TemporalMeasureCS shall have one axis association.
  *
- * \remark Implements TemporalMeasureCS from \ref ISO_19111_2018
+ * \remark Implements TemporalMeasureCS from \ref ISO_19111_2019
  */
 class PROJ_GCC_DLL TemporalMeasureCS final : public TemporalCS {
   public:
@@ -726,7 +748,7 @@ class PROJ_GCC_DLL TemporalMeasureCS final : public TemporalCS {
         const CoordinateSystemAxisNNPtr &axis);
     INLINED_MAKE_SHARED
 
-    PROJ_INTERNAL std::string getWKT2Type(bool use2018Keywords) const override;
+    PROJ_INTERNAL std::string getWKT2Type(bool use2019Keywords) const override;
 
   private:
     TemporalMeasureCS(const TemporalMeasureCS &other) = delete;
