@@ -9,6 +9,8 @@
 #include "emess.h"
 #include "utils.h"
 
+#include <vector>
+
 #if defined(MSDOS) || defined(OS2) || defined(WIN32) || defined(__WIN32__)
 #  include <fcntl.h>
 #  include <io.h>
@@ -19,7 +21,7 @@
 
 #define MAX_LINE 1000
 #define MAX_PARGS 100
-#define PJ_INVERS(P) (P->inv ? 1 : 0)
+#define PJ_INVERSE(P) (P->inv ? 1 : 0)
 
 static PJ *Proj;
 static union {
@@ -113,7 +115,8 @@ static void process(FILE *fid) {
                 facs_bad = proj_errno(Proj);
             }
 
-            data.xy = (*proj.fwd)(data.lp, Proj);
+            const auto xy = (*proj.fwd)(data.lp, Proj);
+            data.xy = xy;
 
             if (dofactors && inverse) {
                 facs = proj_factors(Proj, coord);
@@ -217,7 +220,7 @@ static void vprocess(FILE *fid) {
             linvers = inverse;
 
         if (linvers) {
-            if (!PJ_INVERS(Proj)) {
+            if (!PJ_INVERSE(Proj)) {
                 emess(-1,"inverse for this projection not avail.\n");
                 continue;
             }
@@ -250,10 +253,8 @@ static void vprocess(FILE *fid) {
             if (postscale) { dat_xy.x *= fscale; dat_xy.y *= fscale; }
         }
 
-        /* For some reason pj_errno does not work as expected in some   */
-        /* versions of Visual Studio, so using pj_get_errno_ref instead */
-        if (*pj_get_errno_ref()) {
-            emess(-1, pj_strerrno(*pj_get_errno_ref()));
+        if (proj_context_errno(nullptr)) {
+            emess(-1, proj_errno_string(proj_context_errno(nullptr)));
             continue;
         }
 
@@ -294,15 +295,16 @@ static void vprocess(FILE *fid) {
 
 int main(int argc, char **argv) {
     char *arg;
-    char *pargv[MAX_PARGS] = {};
+    std::vector<char*> argvVector;
     char **eargv = argv;
     FILE *fid;
-    int pargc = 0, eargc = 0, mon = 0;
+    int eargc = 0, mon = 0;
 
     if ( (emess_dat.Prog_name = strrchr(*argv,DIR_CHAR)) != nullptr)
         ++emess_dat.Prog_name;
     else emess_dat.Prog_name = *argv;
-    inverse = ! strncmp(emess_dat.Prog_name, "inv", 3);
+    inverse = strncmp(emess_dat.Prog_name, "inv", 3) == 0 ||
+              strncmp(emess_dat.Prog_name, "lt-inv", 6) == 0; // older libtool have a lt- prefix
     if (argc <= 1 ) {
         (void)fprintf(stderr, usage, pj_get_release(), emess_dat.Prog_name);
         exit (0);
@@ -452,10 +454,7 @@ int main(int argc, char **argv) {
             }
             break;
         } else if (**argv == '+') { /* + argument */
-            if (pargc < MAX_PARGS)
-                pargv[pargc++] = *argv + 1;
-            else
-                emess(1,"overflowed + argument table");
+            argvVector.push_back(*argv + 1);
         } else /* assumed to be input file name(s) */
             eargv[eargc++] = *argv;
     }
@@ -475,9 +474,14 @@ int main(int argc, char **argv) {
         postscale = 0;
         fscale = 1./fscale;
     }
-    if (!(Proj = pj_init(pargc, pargv)))
+    proj_context_use_proj4_init_rules(nullptr, true);
+
+    // proj historically ignores any datum shift specifier, like nadgrids, towgs84, etc
+    argvVector.push_back(const_cast<char*>("break_cs2cs_recursion"));
+
+    if (!(Proj = proj_create_argv(nullptr, static_cast<int>(argvVector.size()), argvVector.data())))
         emess(3,"projection initialization failure\ncause: %s",
-              pj_strerrno(pj_errno));
+              proj_errno_string(proj_context_errno(nullptr)));
 
     if (!proj_angular_input(Proj, PJ_FWD)) {
         emess(3, "can't initialize operations that take non-angular input coordinates");
@@ -562,7 +566,7 @@ int main(int argc, char **argv) {
     }
 
     if( Proj )
-        pj_free(Proj);
+        proj_destroy(Proj);
 
     exit(0); /* normal completion */
 }
