@@ -33,6 +33,7 @@
 #include <cstdlib>
 #include <fstream> // std::ifstream
 #include <iostream>
+#include <set>
 #include <utility>
 
 #include "proj.h"
@@ -68,11 +69,15 @@ struct OutputOptions {
     bool WKT1_GDAL = false;
     bool WKT1_ESRI = false;
     bool PROJJSON = false;
+    bool SQL = false;
     bool c_ify = false;
     bool singleLine = false;
     bool strict = true;
     bool ballparkAllowed = true;
     bool allowEllipsoidalHeightAsVerticalCRS = false;
+    std::string outputAuthName{};
+    std::string outputCode{};
+    std::vector<std::string> allowedAuthorities{};
 };
 } // anonymous namespace
 
@@ -101,24 +106,38 @@ static void usage() {
         << "                [--allow-ellipsoidal-height-as-vertical-crs]"
         << std::endl
         << "                [--boundcrs-to-wgs84]" << std::endl
+        << "                [--authority name]" << std::endl
         << "                [--main-db-path path] [--aux-db-path path]*"
         << std::endl
         << "                [--identify] [--3d]" << std::endl
+        << "                [--output-id AUTH:CODE]" << std::endl
         << "                [--c-ify] [--single-line]" << std::endl
         << "                --searchpaths | --remote-data |" << std::endl
-        << "                {object_definition} | (-s {srs_def} -t {srs_def})"
+        << "                --list-crs [list-crs-filter] |" << std::endl
+        << "                --dump-db-structure [{object_definition} | "
+           "{object_reference}] |"
+        << std::endl
+        << "                {object_definition} | {object_reference} | "
+           "(-s {srs_def} -t {srs_def})"
         << std::endl;
     std::cerr << std::endl;
     std::cerr << "-o: formats is a comma separated combination of: "
                  "all,default,PROJ,WKT_ALL,WKT2:2015,WKT2:2019,WKT1:GDAL,"
-                 "WKT1:ESRI,PROJJSON"
+                 "WKT1:ESRI,PROJJSON,SQL"
               << std::endl;
     std::cerr << "    Except 'all' and 'default', other format can be preceded "
                  "by '-' to disable them"
               << std::endl;
     std::cerr << std::endl;
+    std::cerr << "list-crs-filter is a comma separated combination of: "
+                 "allow_deprecated,geodetic,geocentric,"
+              << std::endl;
+    std::cerr
+        << "geographic,geographic_2d,geographic_3d,vertical,projected,compound"
+        << std::endl;
+    std::cerr << std::endl;
     std::cerr << "{object_definition} might be a PROJ string, a WKT string, "
-                 " a AUTHORITY:CODE, or urn:ogc:def:OBJECT_TYPE:AUTHORITY::CODE"
+                 "a AUTHORITY:CODE, or urn:ogc:def:OBJECT_TYPE:AUTHORITY::CODE"
               << std::endl;
     std::exit(1);
 }
@@ -147,12 +166,98 @@ static std::string c_ify_string(const std::string &str) {
 
 // ---------------------------------------------------------------------------
 
+static ExtentPtr makeBboxFilter(DatabaseContextPtr dbContext,
+                                const std::string &bboxStr,
+                                const std::string &area,
+                                bool errorIfSeveralAreaMatches) {
+    ExtentPtr bboxFilter = nullptr;
+    if (!bboxStr.empty()) {
+        auto bbox(split(bboxStr, ','));
+        if (bbox.size() != 4) {
+            std::cerr << "Incorrect number of values for option --bbox: "
+                      << bboxStr << std::endl;
+            usage();
+        }
+        try {
+            std::vector<double> bboxValues = {
+                c_locale_stod(bbox[0]), c_locale_stod(bbox[1]),
+                c_locale_stod(bbox[2]), c_locale_stod(bbox[3])};
+            bboxFilter = Extent::createFromBBOX(bboxValues[0], bboxValues[1],
+                                                bboxValues[2], bboxValues[3])
+                             .as_nullable();
+        } catch (const std::exception &e) {
+            std::cerr << "Invalid value for option --bbox: " << bboxStr << ", "
+                      << e.what() << std::endl;
+            usage();
+        }
+    } else if (!area.empty()) {
+        assert(dbContext);
+        try {
+            if (area.find(' ') == std::string::npos &&
+                area.find(':') != std::string::npos) {
+                auto tokens = split(area, ':');
+                if (tokens.size() == 2) {
+                    const std::string &areaAuth = tokens[0];
+                    const std::string &areaCode = tokens[1];
+                    bboxFilter = AuthorityFactory::create(
+                                     NN_NO_CHECK(dbContext), areaAuth)
+                                     ->createExtent(areaCode)
+                                     .as_nullable();
+                }
+            }
+            if (!bboxFilter) {
+                auto authFactory = AuthorityFactory::create(
+                    NN_NO_CHECK(dbContext), std::string());
+                auto res = authFactory->listAreaOfUseFromName(area, false);
+                if (res.size() == 1) {
+                    bboxFilter = AuthorityFactory::create(
+                                     NN_NO_CHECK(dbContext), res.front().first)
+                                     ->createExtent(res.front().second)
+                                     .as_nullable();
+                } else {
+                    res = authFactory->listAreaOfUseFromName(area, true);
+                    if (res.size() == 1) {
+                        bboxFilter =
+                            AuthorityFactory::create(NN_NO_CHECK(dbContext),
+                                                     res.front().first)
+                                ->createExtent(res.front().second)
+                                .as_nullable();
+                    } else if (res.empty()) {
+                        std::cerr << "No area of use matching provided name"
+                                  << std::endl;
+                        std::exit(1);
+                    } else if (errorIfSeveralAreaMatches) {
+                        std::cerr << "Several candidates area of use "
+                                     "matching provided name :"
+                                  << std::endl;
+                        for (const auto &candidate : res) {
+                            auto obj =
+                                AuthorityFactory::create(NN_NO_CHECK(dbContext),
+                                                         candidate.first)
+                                    ->createExtent(candidate.second);
+                            std::cerr << "  " << candidate.first << ":"
+                                      << candidate.second << " : "
+                                      << *obj->description() << std::endl;
+                        }
+                        std::exit(1);
+                    }
+                }
+            }
+        } catch (const std::exception &e) {
+            std::cerr << "Area of use retrieval failed: " << e.what()
+                      << std::endl;
+            std::exit(1);
+        }
+    }
+    return bboxFilter;
+}
+
 static BaseObjectNNPtr buildObject(
     DatabaseContextPtr dbContext, const std::string &user_string,
     const std::string &kind, const std::string &context,
     bool buildBoundCRSToWGS84,
     CoordinateOperationContext::IntermediateCRSUse allowUseIntermediateCRS,
-    bool promoteTo3D, bool quiet) {
+    bool promoteTo3D, bool normalizeAxisOrder, bool quiet) {
     BaseObjectPtr obj;
 
     std::string l_user_string(user_string);
@@ -169,7 +274,7 @@ static BaseObjectNNPtr buildObject(
             char buffer[256];
             fs.read(buffer, sizeof(buffer));
             l_user_string.append(buffer, static_cast<size_t>(fs.gcount()));
-            if (l_user_string.size() > 100 * 1000) {
+            if (l_user_string.size() > 1000 * 1000) {
                 fs.close();
                 std::cerr << context << ": too big file" << std::endl;
                 std::exit(1);
@@ -300,6 +405,13 @@ static BaseObjectNNPtr buildObject(
         }
     }
 
+    if (normalizeAxisOrder) {
+        auto crs = std::dynamic_pointer_cast<CRS>(obj);
+        if (crs) {
+            obj = crs->normalizeForVisualization().as_nullable();
+        }
+    }
+
     return NN_NO_CHECK(obj);
 }
 
@@ -310,7 +422,7 @@ static void outputObject(
     CoordinateOperationContext::IntermediateCRSUse allowUseIntermediateCRS,
     const OutputOptions &outputOpt) {
 
-    auto identified = dynamic_cast<const IdentifiedObject *>(obj.get());
+    auto identified = nn_dynamic_pointer_cast<IdentifiedObject>(obj);
     if (!outputOpt.quiet && identified && identified->isDeprecated()) {
         std::cout << "Warning: object is deprecated" << std::endl;
         auto crs = dynamic_cast<const CRS *>(obj.get());
@@ -335,7 +447,7 @@ static void outputObject(
 
     auto projStringExportable =
         nn_dynamic_pointer_cast<IPROJStringExportable>(obj);
-    bool alreadyOutputed = false;
+    bool alreadyOutputted = false;
     if (projStringExportable) {
         if (outputOpt.PROJ5) {
             try {
@@ -369,7 +481,7 @@ static void outputObject(
                 std::cerr << "Error when exporting to PROJ string: " << e.what()
                           << std::endl;
             }
-            alreadyOutputed = true;
+            alreadyOutputted = true;
         }
     }
 
@@ -377,14 +489,14 @@ static void outputObject(
     if (wktExportable) {
         if (outputOpt.WKT2_2015) {
             try {
-                if (alreadyOutputed) {
+                if (alreadyOutputted) {
                     std::cout << std::endl;
                 }
                 if (!outputOpt.quiet) {
                     std::cout << "WKT2:2015 string:" << std::endl;
                 }
-                auto formatter =
-                    WKTFormatter::create(WKTFormatter::Convention::WKT2_2015);
+                auto formatter = WKTFormatter::create(
+                    WKTFormatter::Convention::WKT2_2015, dbContext);
                 formatter->setMultiLine(!outputOpt.singleLine);
                 formatter->setStrict(outputOpt.strict);
                 auto wkt = wktExportable->exportToWKT(formatter.get());
@@ -396,19 +508,19 @@ static void outputObject(
                 std::cerr << "Error when exporting to WKT2:2015: " << e.what()
                           << std::endl;
             }
-            alreadyOutputed = true;
+            alreadyOutputted = true;
         }
 
         if (outputOpt.WKT2_2015_SIMPLIFIED) {
             try {
-                if (alreadyOutputed) {
+                if (alreadyOutputted) {
                     std::cout << std::endl;
                 }
                 if (!outputOpt.quiet) {
                     std::cout << "WKT2:2015_SIMPLIFIED string:" << std::endl;
                 }
                 auto formatter = WKTFormatter::create(
-                    WKTFormatter::Convention::WKT2_2015_SIMPLIFIED);
+                    WKTFormatter::Convention::WKT2_2015_SIMPLIFIED, dbContext);
                 if (outputOpt.singleLine) {
                     formatter->setMultiLine(false);
                 }
@@ -422,19 +534,19 @@ static void outputObject(
                 std::cerr << "Error when exporting to WKT2:2015_SIMPLIFIED: "
                           << e.what() << std::endl;
             }
-            alreadyOutputed = true;
+            alreadyOutputted = true;
         }
 
         if (outputOpt.WKT2_2019) {
             try {
-                if (alreadyOutputed) {
+                if (alreadyOutputted) {
                     std::cout << std::endl;
                 }
                 if (!outputOpt.quiet) {
                     std::cout << "WKT2:2019 string:" << std::endl;
                 }
-                auto formatter =
-                    WKTFormatter::create(WKTFormatter::Convention::WKT2_2019);
+                auto formatter = WKTFormatter::create(
+                    WKTFormatter::Convention::WKT2_2019, dbContext);
                 if (outputOpt.singleLine) {
                     formatter->setMultiLine(false);
                 }
@@ -448,19 +560,19 @@ static void outputObject(
                 std::cerr << "Error when exporting to WKT2:2019: " << e.what()
                           << std::endl;
             }
-            alreadyOutputed = true;
+            alreadyOutputted = true;
         }
 
         if (outputOpt.WKT2_2019_SIMPLIFIED) {
             try {
-                if (alreadyOutputed) {
+                if (alreadyOutputted) {
                     std::cout << std::endl;
                 }
                 if (!outputOpt.quiet) {
                     std::cout << "WKT2:2019_SIMPLIFIED string:" << std::endl;
                 }
                 auto formatter = WKTFormatter::create(
-                    WKTFormatter::Convention::WKT2_2019_SIMPLIFIED);
+                    WKTFormatter::Convention::WKT2_2019_SIMPLIFIED, dbContext);
                 if (outputOpt.singleLine) {
                     formatter->setMultiLine(false);
                 }
@@ -474,12 +586,12 @@ static void outputObject(
                 std::cerr << "Error when exporting to WKT2:2019_SIMPLIFIED: "
                           << e.what() << std::endl;
             }
-            alreadyOutputed = true;
+            alreadyOutputted = true;
         }
 
         if (outputOpt.WKT1_GDAL && !nn_dynamic_pointer_cast<Conversion>(obj)) {
             try {
-                if (alreadyOutputed) {
+                if (alreadyOutputted) {
                     std::cout << std::endl;
                 }
                 if (!outputOpt.quiet) {
@@ -504,12 +616,12 @@ static void outputObject(
                 std::cerr << "Error when exporting to WKT1:GDAL: " << e.what()
                           << std::endl;
             }
-            alreadyOutputed = true;
+            alreadyOutputted = true;
         }
 
         if (outputOpt.WKT1_ESRI && !nn_dynamic_pointer_cast<Conversion>(obj)) {
             try {
-                if (alreadyOutputed) {
+                if (alreadyOutputted) {
                     std::cout << std::endl;
                 }
                 if (!outputOpt.quiet) {
@@ -529,7 +641,7 @@ static void outputObject(
                 std::cerr << "Error when exporting to WKT1:ESRI: " << e.what()
                           << std::endl;
             }
-            alreadyOutputed = true;
+            alreadyOutputted = true;
         }
     }
 
@@ -537,7 +649,7 @@ static void outputObject(
     if (JSONExportable) {
         if (outputOpt.PROJJSON) {
             try {
-                if (alreadyOutputed) {
+                if (alreadyOutputted) {
                     std::cout << std::endl;
                 }
                 if (!outputOpt.quiet) {
@@ -556,10 +668,38 @@ static void outputObject(
                 std::cerr << "Error when exporting to PROJJSON: " << e.what()
                           << std::endl;
             }
-            // alreadyOutputed = true;
+            alreadyOutputted = true;
         }
     }
 
+    if (identified && dbContext && outputOpt.SQL) {
+        try {
+            if (alreadyOutputted) {
+                std::cout << std::endl;
+            }
+            if (!outputOpt.quiet) {
+                std::cout << "SQL:" << std::endl;
+            }
+            dbContext->startInsertStatementsSession();
+            auto allowedAuthorities(outputOpt.allowedAuthorities);
+            if (allowedAuthorities.empty()) {
+                allowedAuthorities.emplace_back("EPSG");
+                allowedAuthorities.emplace_back("PROJ");
+            }
+            const auto statements = dbContext->getInsertStatementsFor(
+                NN_NO_CHECK(identified), outputOpt.outputAuthName,
+                outputOpt.outputCode, false, allowedAuthorities);
+            dbContext->stopInsertStatementsSession();
+            for (const auto &sql : statements) {
+                std::cout << sql << std::endl;
+            }
+        } catch (const std::exception &e) {
+            std::cerr << "Error when exporting to SQL: " << e.what()
+                      << std::endl;
+        }
+        // alreadyOutputted = true;
+    }
+
     auto op = dynamic_cast<CoordinateOperation *>(obj.get());
     if (!outputOpt.quiet && op && dbContext &&
         getenv("PROJINFO_NO_GRID_CHECK") == nullptr) {
@@ -674,12 +814,12 @@ static void outputOperations(
     CoordinateOperationContext::IntermediateCRSUse allowUseIntermediateCRS,
     const std::vector<std::pair<std::string, std::string>> &pivots,
     const std::string &authority, bool usePROJGridAlternatives,
-    bool showSuperseded, bool promoteTo3D, double minimumAccuracy,
-    const OutputOptions &outputOpt, bool summary) {
+    bool showSuperseded, bool promoteTo3D, bool normalizeAxisOrder,
+    double minimumAccuracy, const OutputOptions &outputOpt, bool summary) {
     auto sourceObj =
         buildObject(dbContext, sourceCRSStr, "crs", "source CRS", false,
                     CoordinateOperationContext::IntermediateCRSUse::NEVER,
-                    promoteTo3D, outputOpt.quiet);
+                    promoteTo3D, normalizeAxisOrder, outputOpt.quiet);
     auto sourceCRS = nn_dynamic_pointer_cast<CRS>(sourceObj);
     if (!sourceCRS) {
         std::cerr << "source CRS string is not a CRS" << std::endl;
@@ -690,7 +830,7 @@ static void outputOperations(
     auto targetObj =
         buildObject(dbContext, targetCRSStr, "crs", "target CRS", false,
                     CoordinateOperationContext::IntermediateCRSUse::NEVER,
-                    promoteTo3D, outputOpt.quiet);
+                    promoteTo3D, normalizeAxisOrder, outputOpt.quiet);
     auto targetCRS = nn_dynamic_pointer_cast<CRS>(targetObj);
     if (!targetCRS) {
         std::cerr << "target CRS string is not a CRS" << std::endl;
@@ -796,11 +936,11 @@ int main(int argc, char **argv) {
     bool user_string_specified = false;
     std::string sourceCRSStr;
     std::string targetCRSStr;
-    bool outputSwithSpecified = false;
+    bool outputSwitchSpecified = false;
     OutputOptions outputOpt;
     std::string objectKind;
     bool summary = false;
-    ExtentPtr bboxFilter = nullptr;
+    std::string bboxStr;
     std::string area;
     bool spatialCriterionExplicitlySpecified = false;
     CoordinateOperationContext::SpatialCriterion spatialCriterion =
@@ -824,22 +964,29 @@ int main(int argc, char **argv) {
     bool identify = false;
     bool showSuperseded = false;
     bool promoteTo3D = false;
+    bool normalizeAxisOrder = false;
     double minimumAccuracy = -1;
+    bool outputAll = false;
+    bool dumpDbStructure = false;
+    std::string listCRSFilter;
+    bool listCRSSpecified = false;
 
     for (int i = 1; i < argc; i++) {
         std::string arg(argv[i]);
         if (arg == "-o" && i + 1 < argc) {
-            outputSwithSpecified = true;
+            outputSwitchSpecified = true;
             i++;
             auto formats(split(argv[i], ','));
             for (auto format : formats) {
                 if (ci_equal(format, "all")) {
+                    outputAll = true;
                     outputOpt.PROJ5 = true;
                     outputOpt.WKT2_2019 = true;
                     outputOpt.WKT2_2015 = true;
                     outputOpt.WKT1_GDAL = true;
                     outputOpt.WKT1_ESRI = true;
                     outputOpt.PROJJSON = true;
+                    outputOpt.SQL = true;
                 } else if (ci_equal(format, "default")) {
                     outputOpt.PROJ5 = true;
                     outputOpt.WKT2_2019 = true;
@@ -915,6 +1062,10 @@ int main(int argc, char **argv) {
                     outputOpt.PROJJSON = true;
                 } else if (ci_equal(format, "-PROJJSON")) {
                     outputOpt.PROJJSON = false;
+                } else if (ci_equal(format, "SQL")) {
+                    outputOpt.SQL = true;
+                } else if (ci_equal(format, "-SQL")) {
+                    outputOpt.SQL = false;
                 } else {
                     std::cerr << "Unrecognized value for option -o: " << format
                               << std::endl;
@@ -923,23 +1074,7 @@ int main(int argc, char **argv) {
             }
         } else if (arg == "--bbox" && i + 1 < argc) {
             i++;
-            auto bboxStr(argv[i]);
-            auto bbox(split(bboxStr, ','));
-            if (bbox.size() != 4) {
-                std::cerr << "Incorrect number of values for option --bbox: "
-                          << bboxStr << std::endl;
-                usage();
-            }
-            try {
-                bboxFilter = Extent::createFromBBOX(
-                                 c_locale_stod(bbox[0]), c_locale_stod(bbox[1]),
-                                 c_locale_stod(bbox[2]), c_locale_stod(bbox[3]))
-                                 .as_nullable();
-            } catch (const std::exception &e) {
-                std::cerr << "Invalid value for option --bbox: " << bboxStr
-                          << ", " << e.what() << std::endl;
-                usage();
-            }
+            bboxStr = argv[i];
         } else if (arg == "--accuracy" && i + 1 < argc) {
             i++;
             try {
@@ -1084,6 +1219,7 @@ int main(int argc, char **argv) {
         } else if (arg == "--authority" && i + 1 < argc) {
             i++;
             authority = argv[i];
+            outputOpt.allowedAuthorities = split(authority, ',');
         } else if (arg == "--identify") {
             identify = true;
         } else if (arg == "--show-superseded") {
@@ -1096,6 +1232,27 @@ int main(int argc, char **argv) {
             outputOpt.ballparkAllowed = false;
         } else if (ci_equal(arg, "--3d")) {
             promoteTo3D = true;
+        } else if (ci_equal(arg, "--normalize-axis-order")) {
+            // Undocumented for now
+            normalizeAxisOrder = true;
+        } else if (arg == "--output-id" && i + 1 < argc) {
+            i++;
+            const auto tokens = split(argv[i], ':');
+            if (tokens.size() != 2) {
+                std::cerr << "Invalid value for option --output-id"
+                          << std::endl;
+                usage();
+            }
+            outputOpt.outputAuthName = tokens[0];
+            outputOpt.outputCode = tokens[1];
+        } else if (arg == "--dump-db-structure") {
+            dumpDbStructure = true;
+        } else if (arg == "--list-crs") {
+            listCRSSpecified = true;
+            if (i + 1 < argc && argv[i + 1][0] != '-') {
+                i++;
+                listCRSFilter = argv[i];
+            }
         } else if (ci_equal(arg, "--searchpaths")) {
 #ifdef _WIN32
             constexpr char delim = ';';
@@ -1140,17 +1297,38 @@ int main(int argc, char **argv) {
         }
     }
 
-    if (bboxFilter && !area.empty()) {
+    if (!bboxStr.empty() && !area.empty()) {
         std::cerr << "ERROR: --bbox and --area are exclusive" << std::endl;
         std::exit(1);
     }
 
+    if (dumpDbStructure && user_string_specified && !outputSwitchSpecified) {
+        // Implicit settings in --output-db-structure mode + object
+        outputSwitchSpecified = true;
+        outputOpt.SQL = true;
+        outputOpt.quiet = true;
+    }
+    if (outputOpt.SQL && outputOpt.outputAuthName.empty()) {
+        if (outputAll) {
+            outputOpt.SQL = false;
+            std::cerr << "WARNING: SQL output disable since "
+                         "--output-id AUTH:CODE has not been specified."
+                      << std::endl;
+        } else {
+            std::cerr << "ERROR: --output-id AUTH:CODE must be specified when "
+                         "SQL output is enabled."
+                      << std::endl;
+            std::exit(1);
+        }
+    }
+
     DatabaseContextPtr dbContext;
     try {
         dbContext =
             DatabaseContext::create(mainDBPath, auxDBPath).as_nullable();
     } catch (const std::exception &e) {
-        if (!mainDBPath.empty() || !auxDBPath.empty() || !area.empty()) {
+        if (!mainDBPath.empty() || !auxDBPath.empty() || !area.empty() ||
+            dumpDbStructure) {
             std::cerr << "ERROR: Cannot create database connection: "
                       << e.what() << std::endl;
             std::exit(1);
@@ -1159,6 +1337,119 @@ int main(int argc, char **argv) {
                   << std::endl;
     }
 
+    if (dumpDbStructure) {
+        assert(dbContext);
+        try {
+            const auto structure = dbContext->getDatabaseStructure();
+            for (const auto &sql : structure) {
+                std::cout << sql << std::endl;
+            }
+        } catch (const std::exception &e) {
+            std::cerr << "ERROR: getDatabaseStructure() failed: " << e.what()
+                      << std::endl;
+            std::exit(1);
+        }
+    }
+
+    if (listCRSSpecified) {
+        bool allow_deprecated = false;
+        std::set<AuthorityFactory::ObjectType> types;
+        auto tokens = split(listCRSFilter, ',');
+        if (listCRSFilter.empty()) {
+            tokens.clear();
+        }
+        for (const auto &token : tokens) {
+            if (ci_equal(token, "allow_deprecated")) {
+                allow_deprecated = true;
+            } else if (ci_equal(token, "geodetic")) {
+                types.insert(AuthorityFactory::ObjectType::GEOGRAPHIC_2D_CRS);
+                types.insert(AuthorityFactory::ObjectType::GEOGRAPHIC_3D_CRS);
+                types.insert(AuthorityFactory::ObjectType::GEOCENTRIC_CRS);
+            } else if (ci_equal(token, "geocentric")) {
+                types.insert(AuthorityFactory::ObjectType::GEOCENTRIC_CRS);
+            } else if (ci_equal(token, "geographic")) {
+                types.insert(AuthorityFactory::ObjectType::GEOGRAPHIC_2D_CRS);
+                types.insert(AuthorityFactory::ObjectType::GEOGRAPHIC_3D_CRS);
+            } else if (ci_equal(token, "geographic_2d")) {
+                types.insert(AuthorityFactory::ObjectType::GEOGRAPHIC_2D_CRS);
+            } else if (ci_equal(token, "geographic_3d")) {
+                types.insert(AuthorityFactory::ObjectType::GEOGRAPHIC_3D_CRS);
+            } else if (ci_equal(token, "vertical")) {
+                types.insert(AuthorityFactory::ObjectType::VERTICAL_CRS);
+            } else if (ci_equal(token, "projected")) {
+                types.insert(AuthorityFactory::ObjectType::PROJECTED_CRS);
+            } else if (ci_equal(token, "compound")) {
+                types.insert(AuthorityFactory::ObjectType::COMPOUND_CRS);
+            } else {
+                std::cerr << "Unrecognized value for option --list-crs: "
+                          << token << std::endl;
+                usage();
+            }
+        }
+
+        const std::string areaLower = tolower(area);
+        // If the area name has more than a single match, we
+        // will do filtering on info.areaName
+        auto bboxFilter = makeBboxFilter(dbContext, bboxStr, area, false);
+        auto allowedAuthorities(outputOpt.allowedAuthorities);
+        if (allowedAuthorities.empty()) {
+            allowedAuthorities.emplace_back(std::string());
+        }
+        for (const auto &auth_name : allowedAuthorities) {
+            try {
+                auto actualAuthNames =
+                    dbContext->getVersionedAuthoritiesFromName(auth_name);
+                if (actualAuthNames.empty())
+                    actualAuthNames.push_back(auth_name);
+                for (const auto &actualAuthName : actualAuthNames) {
+                    auto factory = AuthorityFactory::create(
+                        NN_NO_CHECK(dbContext), actualAuthName);
+                    const auto list = factory->getCRSInfoList();
+                    for (const auto &info : list) {
+                        if (!allow_deprecated && info.deprecated) {
+                            continue;
+                        }
+                        if (!types.empty() &&
+                            types.find(info.type) == types.end()) {
+                            continue;
+                        }
+                        if (bboxFilter) {
+                            if (!info.bbox_valid) {
+                                continue;
+                            }
+                            auto crsExtent = Extent::createFromBBOX(
+                                info.west_lon_degree, info.south_lat_degree,
+                                info.east_lon_degree, info.north_lat_degree);
+                            if (spatialCriterion ==
+                                CoordinateOperationContext::SpatialCriterion::
+                                    STRICT_CONTAINMENT) {
+                                if (!bboxFilter->contains(crsExtent)) {
+                                    continue;
+                                }
+                            } else {
+                                if (!bboxFilter->intersects(crsExtent)) {
+                                    continue;
+                                }
+                            }
+                        } else if (!area.empty() &&
+                                   tolower(info.areaName).find(areaLower) ==
+                                       std::string::npos) {
+                            continue;
+                        }
+                        std::cout << info.authName << ":" << info.code << " \""
+                                  << info.name << "\""
+                                  << (info.deprecated ? " [deprecated]" : "")
+                                  << std::endl;
+                    }
+                }
+            } catch (const std::exception &e) {
+                std::cerr << "ERROR: list-crs failed with: " << e.what()
+                          << std::endl;
+                std::exit(1);
+            }
+        }
+    }
+
     if (!sourceCRSStr.empty() && targetCRSStr.empty()) {
         std::cerr << "Source CRS specified, but missing target CRS"
                   << std::endl;
@@ -1173,11 +1464,14 @@ int main(int argc, char **argv) {
             usage();
         }
     } else if (!user_string_specified) {
+        if (dumpDbStructure || listCRSSpecified) {
+            std::exit(0);
+        }
         std::cerr << "Missing user string" << std::endl;
         usage();
     }
 
-    if (!outputSwithSpecified) {
+    if (!outputSwitchSpecified) {
         outputOpt.PROJ5 = true;
         outputOpt.WKT2_2019 = true;
     }
@@ -1186,7 +1480,7 @@ int main(int argc, char **argv) {
         (outputOpt.PROJ5 + outputOpt.WKT2_2019 +
          outputOpt.WKT2_2019_SIMPLIFIED + outputOpt.WKT2_2015 +
          outputOpt.WKT2_2015_SIMPLIFIED + outputOpt.WKT1_GDAL +
-         outputOpt.WKT1_ESRI + outputOpt.PROJJSON) != 1) {
+         outputOpt.WKT1_ESRI + outputOpt.PROJJSON + outputOpt.SQL) != 1) {
         std::cerr << "-q can only be used with a single output format"
                   << std::endl;
         usage();
@@ -1197,7 +1491,7 @@ int main(int argc, char **argv) {
             auto obj(buildObject(dbContext, user_string, objectKind,
                                  "input string", buildBoundCRSToWGS84,
                                  allowUseIntermediateCRS, promoteTo3D,
-                                 outputOpt.quiet));
+                                 normalizeAxisOrder, outputOpt.quiet));
             if (guessDialect) {
                 auto dialect = WKTParser().guessDialect(user_string);
                 std::cout << "Guessed WKT dialect: ";
@@ -1289,75 +1583,15 @@ int main(int argc, char **argv) {
             std::exit(1);
         }
     } else {
-
-        if (!area.empty()) {
-            assert(dbContext);
-            try {
-                if (area.find(' ') == std::string::npos &&
-                    area.find(':') != std::string::npos) {
-                    auto tokens = split(area, ':');
-                    if (tokens.size() == 2) {
-                        const std::string &areaAuth = tokens[0];
-                        const std::string &areaCode = tokens[1];
-                        bboxFilter = AuthorityFactory::create(
-                                         NN_NO_CHECK(dbContext), areaAuth)
-                                         ->createExtent(areaCode)
-                                         .as_nullable();
-                    }
-                }
-                if (!bboxFilter) {
-                    auto authFactory = AuthorityFactory::create(
-                        NN_NO_CHECK(dbContext), std::string());
-                    auto res = authFactory->listAreaOfUseFromName(area, false);
-                    if (res.size() == 1) {
-                        bboxFilter =
-                            AuthorityFactory::create(NN_NO_CHECK(dbContext),
-                                                     res.front().first)
-                                ->createExtent(res.front().second)
-                                .as_nullable();
-                    } else {
-                        res = authFactory->listAreaOfUseFromName(area, true);
-                        if (res.size() == 1) {
-                            bboxFilter =
-                                AuthorityFactory::create(NN_NO_CHECK(dbContext),
-                                                         res.front().first)
-                                    ->createExtent(res.front().second)
-                                    .as_nullable();
-                        } else if (res.empty()) {
-                            std::cerr << "No area of use matching provided name"
-                                      << std::endl;
-                            std::exit(1);
-                        } else {
-                            std::cerr << "Several candidates area of use "
-                                         "matching provided name :"
-                                      << std::endl;
-                            for (const auto &candidate : res) {
-                                auto obj =
-                                    AuthorityFactory::create(
-                                        NN_NO_CHECK(dbContext), candidate.first)
-                                        ->createExtent(candidate.second);
-                                std::cerr << "  " << candidate.first << ":"
-                                          << candidate.second << " : "
-                                          << *obj->description() << std::endl;
-                            }
-                            std::exit(1);
-                        }
-                    }
-                }
-            } catch (const std::exception &e) {
-                std::cerr << "Area of use retrieval failed: " << e.what()
-                          << std::endl;
-                std::exit(1);
-            }
-        }
-
+        auto bboxFilter = makeBboxFilter(dbContext, bboxStr, area, true);
         try {
-            outputOperations(
-                dbContext, sourceCRSStr, targetCRSStr, bboxFilter,
-                spatialCriterion, spatialCriterionExplicitlySpecified,
-                crsExtentUse, gridAvailabilityUse, allowUseIntermediateCRS,
-                pivots, authority, usePROJGridAlternatives, showSuperseded,
-                promoteTo3D, minimumAccuracy, outputOpt, summary);
+            outputOperations(dbContext, sourceCRSStr, targetCRSStr, bboxFilter,
+                             spatialCriterion,
+                             spatialCriterionExplicitlySpecified, crsExtentUse,
+                             gridAvailabilityUse, allowUseIntermediateCRS,
+                             pivots, authority, usePROJGridAlternatives,
+                             showSuperseded, promoteTo3D, normalizeAxisOrder,
+                             minimumAccuracy, outputOpt, summary);
         } catch (const std::exception &e) {
             std::cerr << "outputOperations() failed with: " << e.what()
                       << std::endl;
