@@ -1,7 +1,7 @@
 /******************************************************************************
  * Project:  PROJ.4
  * Purpose:  Initialize projection object from string definition.  Includes
- *           pj_init(), pj_init_plus() and pj_free() function.
+ *           pj_init(), and pj_init_plus() function.
  * Author:   Gerald Evenden, Frank Warmerdam <warmerdam@pobox.com>
  *
  ******************************************************************************
@@ -30,7 +30,6 @@
 #define PJ_LIB__
 
 #include <ctype.h>
-#include <errno.h>
 #include <math.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -42,7 +41,6 @@
 #include "filemanager.hpp"
 #include <math.h>
 
-
 /**************************************************************************************/
 static paralist *string_to_paralist (PJ_CONTEXT *ctx, char *definition) {
 /***************************************************************************************
@@ -55,7 +53,7 @@ static paralist *string_to_paralist (PJ_CONTEXT *ctx, char *definition) {
         /* Keep a handle to the start of the list, so we have something to return */
         auto param = pj_mkparam_ws (c, &c);
         if (nullptr==param) {
-            pj_dealloc_params (ctx, first, ENOMEM);
+            free_params (ctx, first, PROJ_ERR_OTHER /*ENOMEM*/);
             return nullptr;
         }
         if (nullptr==last) {
@@ -84,7 +82,7 @@ static char *get_init_string (PJ_CONTEXT *ctx, const char *name) {
     char *buffer = nullptr;
     size_t n;
 
-    fname = static_cast<char*>(pj_malloc (MAX_PATH_FILENAME+ID_TAG_MAX+3));
+    fname = static_cast<char*>(malloc (MAX_PATH_FILENAME+ID_TAG_MAX+3));
     if (nullptr==fname) {
         return nullptr;
     }
@@ -96,7 +94,7 @@ static char *get_init_string (PJ_CONTEXT *ctx, const char *name) {
     else
         key += 5;
     if (MAX_PATH_FILENAME + ID_TAG_MAX + 2 < strlen (key)) {
-        pj_dealloc (fname);
+        free (fname);
         return nullptr;
     }
     memmove (fname, key, strlen (key) + 1);
@@ -104,8 +102,9 @@ static char *get_init_string (PJ_CONTEXT *ctx, const char *name) {
     /* Locate the name of the section we search for */
     section = strrchr(fname, ':');
     if (nullptr==section) {
-        proj_context_errno_set (ctx, PJD_ERR_NO_COLON_IN_INIT_STRING);
-        pj_dealloc (fname);
+        pj_log(ctx, PJ_LOG_ERROR, _("Missing colon in +init"));
+        proj_context_errno_set (ctx, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        free (fname);
         return nullptr;
     }
     *section = 0;
@@ -117,8 +116,9 @@ static char *get_init_string (PJ_CONTEXT *ctx, const char *name) {
 
     auto file = NS_PROJ::FileManager::open_resource_file(ctx, fname);
     if (nullptr==file) {
-        pj_dealloc (fname);
-        proj_context_errno_set (ctx, PJD_ERR_NO_OPTION_IN_INIT_FILE);
+        pj_log(ctx, PJ_LOG_ERROR, _("Cannot open %s"), fname);
+        proj_context_errno_set (ctx, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        free (fname);
         return nullptr;
     }
 
@@ -131,8 +131,9 @@ static char *get_init_string (PJ_CONTEXT *ctx, const char *name) {
         line = file->read_line(MAX_LINE_LENGTH, maxLenReached, eofReached);
         /* End of file? */
         if (maxLenReached || eofReached) {
-            pj_dealloc (fname);
-            proj_context_errno_set (ctx, PJD_ERR_NO_OPTION_IN_INIT_FILE);
+            pj_log(ctx, PJ_LOG_ERROR, _("Invalid content for %s"), fname);
+            proj_context_errno_set (ctx, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+            free (fname);
             return nullptr;
         }
 
@@ -149,9 +150,9 @@ static char *get_init_string (PJ_CONTEXT *ctx, const char *name) {
     }
 
     /* We're at the first line of the right section - copy line to buffer */
-    buffer = static_cast<char*>(pj_malloc (current_buffer_size));
+    buffer = static_cast<char*>(malloc (current_buffer_size));
     if (nullptr==buffer) {
-        pj_dealloc (fname);
+        free (fname);
         return nullptr;
     }
 
@@ -183,22 +184,22 @@ static char *get_init_string (PJ_CONTEXT *ctx, const char *name) {
         pj_chomp (&line[0]);   /* Remove '#' style comments */
         next_length = strlen (line.data()) + buffer_length + 2;
         if (next_length > current_buffer_size) {
-            char *b = static_cast<char*>(pj_malloc (2 * current_buffer_size));
+            char *b = static_cast<char*>(malloc (2 * current_buffer_size));
             if (nullptr==b) {
-                pj_dealloc (buffer);
+                free (buffer);
                 buffer = nullptr;
                 break;
             }
             strcpy (b, buffer);
             current_buffer_size *= 2;
-            pj_dealloc (buffer);
+            free (buffer);
             buffer = b;
         }
         buffer[buffer_length] = ' ';
         strcpy (buffer + buffer_length + 1, line.data());
     }
 
-    pj_dealloc (fname);
+    free (fname);
     if (nullptr==buffer)
         return nullptr;
     pj_shrink (buffer);
@@ -257,7 +258,7 @@ Expand key from buffer or (if not in buffer) from init file
             PJ* src;
             const char* proj_string;
 
-            pj_ctx_set_errno( ctx, 0 );
+            proj_context_errno_set( ctx, 0 );
 
             if( !allow_init_epsg ) {
                 pj_log (ctx, PJ_LOG_TRACE, "%s expansion disallowed", xkey);
@@ -306,7 +307,7 @@ Expand key from buffer or (if not in buffer) from init file
                 definition,
                 init_items->param,
                 init_items->next ? init_items->next->param : "(empty)");
-    pj_dealloc (definition);
+    free (definition);
     if (nullptr==init_items)
         return nullptr;
 
@@ -399,90 +400,6 @@ paralist *pj_expand_init(PJ_CONTEXT *ctx, paralist *init) {
 }
 
 
-/************************************************************************/
-/*                            pj_init_plus()                            */
-/*                                                                      */
-/*      Same as pj_init() except it takes one argument string with      */
-/*      individual arguments preceded by '+', such as "+proj=utm        */
-/*      +zone=11 +ellps=WGS84".                                         */
-/************************************************************************/
-
-PJ *
-pj_init_plus( const char *definition )
-
-{
-    return pj_init_plus_ctx( pj_get_default_ctx(), definition );
-}
-
-PJ *
-pj_init_plus_ctx( projCtx ctx, const char *definition )
-{
-#define MAX_ARG 200
-    char	*argv[MAX_ARG];
-    char	*defn_copy;
-    int		argc = 0, i, blank_count = 0;
-    PJ	    *result = nullptr;
-
-    /* make a copy that we can manipulate */
-    defn_copy = (char *) pj_malloc( strlen(definition)+1 );
-    if (!defn_copy)
-        return nullptr;
-    strcpy( defn_copy, definition );
-
-    /* split into arguments based on '+' and trim white space */
-
-    for( i = 0; defn_copy[i] != '\0'; i++ )
-    {
-        switch( defn_copy[i] )
-        {
-          case '+':
-            if( i == 0 || defn_copy[i-1] == '\0' || blank_count > 0 )
-            {
-                /* trim trailing spaces from the previous param */
-                if( blank_count > 0 )
-                {
-                    defn_copy[i - blank_count] = '\0';
-                    blank_count = 0;
-                }
-
-                if( argc+1 == MAX_ARG )
-                {
-                    pj_dalloc( defn_copy );
-                    pj_ctx_set_errno( ctx, PJD_ERR_UNPARSEABLE_CS_DEF );
-                    return nullptr;
-                }
-
-                argv[argc++] = defn_copy + i + 1;
-            }
-            break;
-
-          case ' ':
-          case '\t':
-          case '\n':
-            /* trim leading spaces from the current param */
-            if( i == 0 || defn_copy[i-1] == '\0' || argc == 0 || argv[argc-1] == defn_copy + i )
-                defn_copy[i] = '\0';
-            else
-                blank_count++;
-            break;
-
-          default:
-            /* reset blank_count */
-            blank_count = 0;
-        }
-    }
-    /* trim trailing spaces from the last param */
-    defn_copy[i - blank_count] = '\0';
-
-    /* perform actual initialization */
-    result = pj_init_ctx( ctx, argc, argv );
-
-    pj_dalloc( defn_copy );
-    return result;
-}
-
-
-
 /************************************************************************/
 /*                              pj_init()                               */
 /*                                                                      */
@@ -492,11 +409,6 @@ pj_init_plus_ctx( projCtx ctx, const char *definition )
 /*      large enough to hold projection specific parameters.            */
 /************************************************************************/
 
-PJ *
-pj_init(int argc, char **argv) {
-    return pj_init_ctx( pj_get_default_ctx(), argc, argv );
-}
-
 
 static PJ_CONSTRUCTOR locate_constructor (const char *name) {
     int i;
@@ -511,15 +423,7 @@ static PJ_CONSTRUCTOR locate_constructor (const char *name) {
 
 
 PJ *
-pj_init_ctx(projCtx ctx, int argc, char **argv) {
-    /* Legacy interface: allow init=epsg:XXXX syntax by default */
-    int allow_init_epsg = proj_context_get_use_proj4_init_rules(ctx, TRUE);
-    return pj_init_ctx_with_allow_init_epsg(ctx, argc, argv, allow_init_epsg);
-}
-
-
-PJ *
-pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_init_epsg) {
+pj_init_ctx_with_allow_init_epsg(PJ_CONTEXT *ctx, int argc, char **argv, int allow_init_epsg) {
     const char *s;
     char *name;
     PJ_CONSTRUCTOR proj;
@@ -538,7 +442,8 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     ctx->last_errno = 0;
 
     if (argc <= 0) {
-        pj_ctx_set_errno (ctx, PJD_ERR_NO_ARGS);
+        pj_log(ctx, PJ_LOG_ERROR, _("No arguments"));
+        proj_context_errno_set (ctx, PROJ_ERR_INVALID_OP_MISSING_ARG);
         return nullptr;
     }
 
@@ -552,13 +457,15 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
 
     /* can't have nested pipelines directly */
     if (n_pipelines > 1) {
-        pj_ctx_set_errno (ctx, PJD_ERR_MALFORMED_PIPELINE);
+        pj_log(ctx, PJ_LOG_ERROR, _("Nested pipelines are not supported"));
+        proj_context_errno_set (ctx, PROJ_ERR_INVALID_OP_WRONG_SYNTAX);
         return nullptr;
     }
 
     /* don't allow more than one +init in non-pipeline operations */
     if (n_pipelines == 0 && n_inits > 1) {
-        pj_ctx_set_errno (ctx, PJD_ERR_TOO_MANY_INITS);
+        pj_log(ctx, PJ_LOG_ERROR, _("Too many inits"));
+        proj_context_errno_set (ctx, PROJ_ERR_INVALID_OP_WRONG_SYNTAX);
         return nullptr;
     }
 
@@ -566,14 +473,14 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     /* put arguments into internal linked list */
     start = curr = pj_mkparam(argv[0]);
     if (!curr) {
-        pj_dealloc_params (ctx, start, ENOMEM);
+        free_params (ctx, start, PROJ_ERR_OTHER /*ENOMEM*/);
         return nullptr;
     }
 
     for (i = 1; i < argc; ++i) {
         curr->next = pj_mkparam(argv[i]);
         if (!curr->next) {
-            pj_dealloc_params (ctx, start, ENOMEM);
+            free_params (ctx, start, PROJ_ERR_OTHER /*ENOMEM*/);
             return nullptr;
         }
         curr = curr->next;
@@ -588,31 +495,34 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     if (init && n_pipelines == 0) {
         init = pj_expand_init_internal (ctx, init, allow_init_epsg);
         if (!init) {
-            pj_dealloc_params (ctx, start, PJD_ERR_NO_ARGS);
+            free_params (ctx, start, PROJ_ERR_INVALID_OP_WRONG_SYNTAX);
             return nullptr;
         }
     }
     if (ctx->last_errno) {
-        pj_dealloc_params (ctx, start, ctx->last_errno);
+        free_params (ctx, start, ctx->last_errno);
         return nullptr;
     }
 
     /* Find projection selection */
     curr = pj_param_exists (start, "proj");
     if (nullptr==curr) {
-        pj_dealloc_params (ctx, start, PJD_ERR_PROJ_NOT_NAMED);
+        pj_log(ctx, PJ_LOG_ERROR, _("Missing proj"));
+        free_params (ctx, start, PROJ_ERR_INVALID_OP_MISSING_ARG);
         return nullptr;
     }
     name =  curr->param;
     if (strlen (name) < 6) {
-        pj_dealloc_params (ctx, start, PJD_ERR_PROJ_NOT_NAMED);
+        pj_log(ctx, PJ_LOG_ERROR, _("Invalid value for proj"));
+        free_params (ctx, start, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
         return nullptr;
     }
     name += 5;
 
     proj = locate_constructor (name);
     if (nullptr==proj) {
-        pj_dealloc_params (ctx, start, PJD_ERR_UNKNOWN_PROJECTION_ID);
+        pj_log(ctx, PJ_LOG_ERROR, _("Unknown projection"));
+        free_params (ctx, start, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
         return nullptr;
     }
 
@@ -621,7 +531,7 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     /* Allocate projection structure */
     PIN = proj(nullptr);
     if (nullptr==PIN) {
-        pj_dealloc_params (ctx, start, ENOMEM);
+        free_params (ctx, start, PROJ_ERR_OTHER /*ENOMEM*/);
         return nullptr;
     }
 
@@ -647,11 +557,11 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     if (err) {
         /* Didn't get an ellps, but doesn't need one: Get a free WGS84 */
         if (PIN->need_ellps) {
-            pj_log (ctx, PJ_LOG_DEBUG_MINOR, "pj_init_ctx: Must specify ellipsoid or sphere");
+            pj_log (ctx, PJ_LOG_ERROR, _("pj_init_ctx: Must specify ellipsoid or sphere"));
             return pj_default_destructor (PIN, proj_errno(PIN));
         }
         else {
-            if (PJD_ERR_MAJOR_AXIS_NOT_GIVEN==proj_errno (PIN))
+            if (PIN->a == 0)
                 proj_errno_reset (PIN);
             PIN->f = 1.0/298.257223563;
             PIN->a  = 6378137.0;
@@ -661,7 +571,7 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     PIN->a_orig = PIN->a;
     PIN->es_orig = PIN->es;
     if (pj_calc_ellipsoid_params (PIN, PIN->a, PIN->es))
-        return pj_default_destructor (PIN, PJD_ERR_INVALID_ECCENTRICITY);
+        return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
 
     /* Now that we have ellipse information check for WGS84 datum */
     if( PIN->datum_type == PJD_3PARAM
@@ -693,7 +603,10 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
         /* when correcting longitudes around it */
         /* The test is written this way to error on long_wrap_center "=" NaN */
         if( !(fabs(PIN->long_wrap_center) < 10 * M_TWOPI) )
-            return pj_default_destructor (PIN, PJD_ERR_LAT_OR_LON_EXCEED_LIMIT);
+        {
+            proj_log_error(PIN, _("Invalid value for lon_wrap"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
     }
 
     /* Axis orientation */
@@ -702,12 +615,18 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
         const char *axis_legal = "ewnsud";
         const char *axis_arg = pj_param(ctx, start,"saxis").s;
         if( strlen(axis_arg) != 3 )
-            return pj_default_destructor (PIN, PJD_ERR_AXIS);
+        {
+            proj_log_error(PIN, _("Invalid value for axis"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
 
         if( strchr( axis_legal, axis_arg[0] ) == nullptr
             || strchr( axis_legal, axis_arg[1] ) == nullptr
             || strchr( axis_legal, axis_arg[2] ) == nullptr)
-            return pj_default_destructor (PIN, PJD_ERR_AXIS);
+        {
+            proj_log_error(PIN, _("Invalid value for axis"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
 
         /* TODO: it would be nice to validate we don't have on axis repeated */
         strcpy( PIN->axis, axis_arg );
@@ -719,7 +638,10 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     /* Central latitude */
     PIN->phi0 = pj_param(ctx, start, "rlat_0").f;
     if( fabs(PIN->phi0) > M_HALFPI )
-        return pj_default_destructor (PIN, PJD_ERR_LAT_LARGER_THAN_90);
+    {
+        proj_log_error(PIN, _("Invalid value for lat_0: |lat_0| should be <= 90°"));
+        return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+    }
 
     /* False easting and northing */
     PIN->x0 = pj_param(ctx, start, "dx_0").f;
@@ -735,7 +657,10 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     else
         PIN->k0 = 1.;
     if (PIN->k0 <= 0.)
-        return pj_default_destructor (PIN, PJD_ERR_K_LESS_THAN_ZERO);
+    {
+        proj_log_error(PIN, _("Invalid value for k/k_0: it should be > 0"));
+        return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+    }
 
     /* Set units */
     units = pj_list_linear_units();
@@ -743,7 +668,10 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     if ((name = pj_param(ctx, start, "sunits").s) != nullptr) {
         for (i = 0; (s = units[i].id) && strcmp(name, s) ; ++i) ;
         if (!s)
-            return pj_default_destructor (PIN, PJD_ERR_UNKNOWN_UNIT_ID);
+        {
+            proj_log_error(PIN, _("Invalid value for units"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
         s = units[i].to_meter;
     }
     if (s || (s = pj_param(ctx, start, "sto_meter").s)) {
@@ -754,11 +682,17 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
             ++s;
             double denom = pj_strtod(s, nullptr);
             if (denom == 0.0)
-                return pj_default_destructor (PIN, PJD_ERR_UNIT_FACTOR_LESS_THAN_0);
+            {
+                proj_log_error(PIN, _("Invalid value for to_meter donominator"));
+                return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+            }
             PIN->to_meter /= denom;
         }
         if (PIN->to_meter <= 0.0)
-            return pj_default_destructor (PIN, PJD_ERR_UNIT_FACTOR_LESS_THAN_0);
+        {
+            proj_log_error(PIN, _("Invalid value for to_meter"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
         PIN->fr_meter = 1 / PIN->to_meter;
 
     } else
@@ -769,7 +703,10 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
     if ((name = pj_param(ctx, start, "svunits").s) != nullptr) {
         for (i = 0; (s = units[i].id) && strcmp(name, s) ; ++i) ;
         if (!s)
-            return pj_default_destructor (PIN, PJD_ERR_UNKNOWN_UNIT_ID);
+        {
+            proj_log_error(PIN, _("Invalid value for vunits"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
         s = units[i].to_meter;
     }
     if (s || (s = pj_param(ctx, start, "svto_meter").s)) {
@@ -780,11 +717,17 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
             ++s;
             double denom = pj_strtod(s, nullptr);
             if (denom == 0.0)
-                return pj_default_destructor (PIN, PJD_ERR_UNIT_FACTOR_LESS_THAN_0);
+            {
+                proj_log_error(PIN, _("Invalid value for vto_meter donominator"));
+                return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+            }
             PIN->vto_meter /= denom;
         }
         if (PIN->vto_meter <= 0.0)
-            return pj_default_destructor (PIN, PJD_ERR_UNIT_FACTOR_LESS_THAN_0);
+        {
+            proj_log_error(PIN, _("Invalid value for vto_meter"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
         PIN->vfr_meter = 1. / PIN->vto_meter;
     } else {
         PIN->vto_meter = PIN->to_meter;
@@ -813,23 +756,26 @@ pj_init_ctx_with_allow_init_epsg(projCtx ctx, int argc, char **argv, int allow_i
             value = name;
 
         if (!value)
-            return pj_default_destructor (PIN, PJD_ERR_UNKNOWN_PRIME_MERIDIAN);
+        {
+            proj_log_error(PIN, _("Invalid value for pm"));
+            return pj_default_destructor (PIN, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        }
         PIN->from_greenwich = dmstor_ctx(ctx,value,nullptr);
     }
     else
         PIN->from_greenwich = 0.0;
 
     /* Private object for the geodesic functions */
-    PIN->geod = static_cast<struct geod_geodesic*>(pj_calloc (1, sizeof (struct geod_geodesic)));
+    PIN->geod = static_cast<struct geod_geodesic*>(calloc (1, sizeof (struct geod_geodesic)));
     if (nullptr==PIN->geod)
-        return pj_default_destructor (PIN, ENOMEM);
+        return pj_default_destructor (PIN, PROJ_ERR_OTHER /*ENOMEM*/);
     geod_init(PIN->geod, PIN->a,  (1 - sqrt (1 - PIN->es)));
 
     /* Projection specific initialization */
     err = proj_errno_reset (PIN);
     PIN = proj(PIN);
     if (proj_errno (PIN)) {
-        pj_free(PIN);
+        proj_destroy(PIN);
         return nullptr;
     }
     proj_errno_restore (PIN, err);
