@@ -43,7 +43,6 @@
 // clang-format off
 #include "proj.h"
 #include "proj_internal.h"
-#include "proj_api.h"
 // clang-format on
 
 #include "proj_json_streaming_writer.hpp"
@@ -364,6 +363,18 @@ void PrimeMeridian::_exportToWKT(
                     aliasFound = true;
                 }
             }
+            if (!aliasFound && dbContext) {
+                auto authFactory = io::AuthorityFactory::create(
+                    NN_NO_CHECK(dbContext), "ESRI");
+                aliasFound =
+                    authFactory
+                        ->createObjectsFromName(
+                            l_name,
+                            {io::AuthorityFactory::ObjectType::PRIME_MERIDIAN},
+                            false // approximateMatch
+                            )
+                        .size() == 1;
+            }
             if (!aliasFound) {
                 l_name = io::WKTFormatter::morphNameToESRI(l_name);
             }
@@ -435,7 +446,7 @@ std::string
 PrimeMeridian::getPROJStringWellKnownName(const common::Angle &angle) {
     const double valRad = angle.getSIValue();
     std::string projPMName;
-    projCtx ctxt = pj_ctx_alloc();
+    PJ_CONTEXT *ctxt = proj_context_create();
     auto proj_pm = proj_list_prime_meridians();
     for (int i = 0; proj_pm[i].id != nullptr; ++i) {
         double valRefRad = dmstor_ctx(ctxt, proj_pm[i].defn, nullptr);
@@ -444,7 +455,7 @@ PrimeMeridian::getPROJStringWellKnownName(const common::Angle &angle) {
             break;
         }
     }
-    pj_ctx_free(ctxt);
+    proj_context_destroy(ctxt);
     return projPMName;
 }
 //! @endcond
@@ -821,6 +832,18 @@ void Ellipsoid::_exportToWKT(
                             aliasFound = true;
                         }
                     }
+                    if (!aliasFound && dbContext) {
+                        auto authFactory = io::AuthorityFactory::create(
+                            NN_NO_CHECK(dbContext), "ESRI");
+                        aliasFound = authFactory
+                                         ->createObjectsFromName(
+                                             l_name,
+                                             {io::AuthorityFactory::ObjectType::
+                                                  ELLIPSOID},
+                                             false // approximateMatch
+                                             )
+                                         .size() == 1;
+                    }
                     if (!aliasFound) {
                         l_name = io::WKTFormatter::morphNameToESRI(l_name);
                     }
@@ -836,9 +859,8 @@ void Ellipsoid::_exportToWKT(
         }
         formatter->add(computedInverseFlattening());
         const auto &unit = semiMajor.unit();
-        if (isWKT2 &&
-            !(formatter->ellipsoidUnitOmittedIfMetre() &&
-              unit == common::UnitOfMeasure::METRE)) {
+        if (isWKT2 && !(formatter->ellipsoidUnitOmittedIfMetre() &&
+                        unit == common::UnitOfMeasure::METRE)) {
             unit._exportToWKT(formatter, io::WKTConstants::LENGTHUNIT);
         }
         if (formatter->outputId()) {
@@ -1249,6 +1271,18 @@ void GeodeticReferenceFrame::_exportToWKT(
                         }
                     }
                 }
+                if (!aliasFound && dbContext) {
+                    auto authFactory = io::AuthorityFactory::create(
+                        NN_NO_CHECK(dbContext), "ESRI");
+                    aliasFound = authFactory
+                                     ->createObjectsFromName(
+                                         l_name,
+                                         {io::AuthorityFactory::ObjectType::
+                                              GEODETIC_REFERENCE_FRAME},
+                                         false // approximateMatch
+                                         )
+                                     .size() == 1;
+                }
                 if (!aliasFound) {
                     l_name = io::WKTFormatter::morphNameToESRI(l_name);
                     if (!starts_with(l_name, "D_")) {
@@ -1270,8 +1304,9 @@ void GeodeticReferenceFrame::_exportToWKT(
                     // as to trigger the caching done in createObjectsFromName()
                     // in that case.
                     auto matches = factory->createObjectsFromName(
-                        l_name, {io::AuthorityFactory::ObjectType::
-                                     GEODETIC_REFERENCE_FRAME},
+                        l_name,
+                        {io::AuthorityFactory::ObjectType::
+                             GEODETIC_REFERENCE_FRAME},
                         true, 2);
                     if (matches.size() == 1) {
                         const auto &match = matches.front();
@@ -1402,17 +1437,32 @@ bool GeodeticReferenceFrame::hasEquivalentNameToUsingAlias(
     if (dbContext) {
         if (!identifiers().empty()) {
             const auto &id = identifiers().front();
-            auto aliasesResult =
+
+            const std::string officialNameFromId = dbContext->getName(
+                "geodetic_datum", *(id->codeSpace()), id->code());
+            const auto aliasesResult =
                 dbContext->getAliases(*(id->codeSpace()), id->code(), nameStr(),
                                       "geodetic_datum", std::string());
-            const char *otherName = other->nameStr().c_str();
-            for (const auto &aliasResult : aliasesResult) {
-                if (metadata::Identifier::isEquivalentName(
-                        otherName, aliasResult.c_str())) {
-                    return true;
-                }
-            }
-            return false;
+
+            const auto isNameMatching =
+                [&aliasesResult, &officialNameFromId](const std::string &name) {
+                    const char *nameCstr = name.c_str();
+                    if (metadata::Identifier::isEquivalentName(
+                            nameCstr, officialNameFromId.c_str())) {
+                        return true;
+                    } else {
+                        for (const auto &aliasResult : aliasesResult) {
+                            if (metadata::Identifier::isEquivalentName(
+                                    nameCstr, aliasResult.c_str())) {
+                                return true;
+                            }
+                        }
+                    }
+                    return false;
+                };
+
+            return isNameMatching(nameStr()) &&
+                   isNameMatching(other->nameStr());
         } else if (!other->identifiers().empty()) {
             auto otherGRF = dynamic_cast<const GeodeticReferenceFrame *>(other);
             if (otherGRF) {
@@ -1748,6 +1798,14 @@ void DatumEnsemble::_exportToWKT(
     formatter->startNode(io::WKTConstants::ENSEMBLEACCURACY, false);
     formatter->add(positionalAccuracy()->value());
     formatter->endNode();
+
+    // In theory, we should do the following, but currently the WKT grammar
+    // doesn't allow this
+    // ObjectUsage::baseExportToWKT(formatter);
+    if (formatter->outputId()) {
+        formatID(formatter);
+    }
+
     formatter->endNode();
 }
 //! @endcond
@@ -1863,8 +1921,8 @@ RealizationMethod::RealizationMethod(const std::string &nameIn)
 
 // ---------------------------------------------------------------------------
 
-RealizationMethod &RealizationMethod::
-operator=(const RealizationMethod &other) {
+RealizationMethod &
+RealizationMethod::operator=(const RealizationMethod &other) {
     CodeList::operator=(other);
     return *this;
 }
@@ -1963,6 +2021,18 @@ void VerticalReferenceFrame::_exportToWKT(
                     aliasFound = true;
                 }
             }
+            if (!aliasFound && dbContext) {
+                auto authFactory = io::AuthorityFactory::create(
+                    NN_NO_CHECK(dbContext), "ESRI");
+                aliasFound = authFactory
+                                 ->createObjectsFromName(
+                                     l_name,
+                                     {io::AuthorityFactory::ObjectType::
+                                          VERTICAL_REFERENCE_FRAME},
+                                     false // approximateMatch
+                                     )
+                                 .size() == 1;
+            }
             if (!aliasFound) {
                 l_name = io::WKTFormatter::morphNameToESRI(l_name);
             }
